unit Timeodb;
(*
 Timeout Database V1.02 - released June 10, 1998
 Copyright (c) 1997-1998 by Demian
 Requires: ETimer.pas unit (supplied).

 Disclaimer:
 This component is distributed as Freeware. There is no charge or warranty
 whatsoever. The author will not be held responsible for any direct or indirect
 damage resulting from the use of the component or any derivatives thereof.
 In short: USE IT AT YOUR OWN RISK! You are allowed to include the component
 code in any commercial or non commercial product. You are also allowed to make
 as many copies of the component as you want and distribute them, as long as you
 do not receive any money for the copying/distribution.

 You can send any comments, suggestions and bugs to demian@bhnet.com.br
 New versions can be found at: http://www.bhnet.com.br/~demian
                               http://web.unix.horizontes.com.br/~demian

 If you do any modification to this code, please send me a copy.
 If you use this code on any 'real-life' application, please let me know.
 If you think this code is completely useless, please DON'T LET ME KNOW!

 What it does:
 TDatabase descendent that monitors its child datasets and releases any
 pending edits idle for more than Timeout period (default to 1 minute).

 How to use it:
 1. Install the component [VNM Pallete];
 2. Drop a TTimeoutDatabase component onto your application main form;
 3. Set the Timeout property;
 3. Drop any number of TDatasets (TTable/TQuery) onto a form and connect them
    to the  TTimeoutDatabase.

 History:
 V1.00: Created;
 V1.01: One instance of TTimer per application;
 V1.02: Use of Enhanced Timer V1.00 (TETimer component);
*)

interface

uses
  Classes,DB,DBTables,WinProcs,Wintypes,ExtCtrls,ETimer;

type
  TTimeoutDatabase = class(TDatabase)
  private
    FTimeout: LongInt;
    FTimeKeeper: TETimer;
    procedure SetTimeout(Value: LongInt);
    procedure ResetIdle(Sender: TObject);
  protected
  public
    constructor Create(AOwner: TComponent); override;
  published
    Property Timeout: LongInt read FTimeout write SetTimeout;
end;

procedure Register;

implementation

{$IFDEF WIN32}
 {$R *.R32}
{$ELSE}
 {$R *.R16}
{$ENDIF}

{______________________________________________________________________________}
procedure Register;
begin
  RegisterComponents('VNM',[TTimeoutDatabase]);
end;

{______________________________________________________________________________}
constructor TTimeoutDatabase.Create(AOwner:TComponent);
begin
  inherited Create(AOwner);
  FTimeKeeper := TETimer.Create(self);
  FTimeKeeper.OnTimer := ResetIdle;
end;

{______________________________________________________________________________}
procedure TTimeoutDatabase.SetTimeout(Value: LongInt);
begin
  FTimeout := Value;
  Value := Value div 3;
  if Value > 65535 then
    Value := 65535;
  FTimeKeeper.Interval := Value;
end;

{______________________________________________________________________________}
procedure TTimeoutDatabase.ResetIdle(Sender: TObject);
var
  i: integer;
begin
  with TETimer(Sender) do
  if Snooze > FTimeout then begin
    for i := DatasetCount-1 downto 0 do
      try
        Datasets[i].Cancel;
      except
      end;
    Snooze := 0;
  end;
end;

end.

